// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file ODirSolver.h */

#ifndef AMDIS_ODIRSOLVER_H
#define AMDIS_ODIRSOLVER_H

#include "OEMSolver.h"
#include "MemoryManager.h"

namespace AMDiS {

  // ============================================================================
  // ===== class BiCGSolver =====================================================
  // ============================================================================

  /**
   * \ingroup Solver
   * 
   * \brief
   * Solves a linear system by the method of orthogonal directions and can be
   * used for symmetric positive system matrices.
   */
  template<typename VectorType>
  class ODirSolver : public OEMSolver<VectorType>
  {
  public:
    MEMORY_MANAGED(ODirSolver<VectorType>);

    /** \brief
     * Creator class used in the OEMSolverMap.
     */
    class Creator : public OEMSolverCreator<VectorType>
    {
    public:
      MEMORY_MANAGED(Creator);

      virtual ~Creator() {};

      /** \brief
       * Returns a new ODirSolver object.
       */   
      OEMSolver<VectorType>* create() { 
	return NEW ODirSolver<VectorType>(this->name);
      };
    };

    /** \brief
     * constructor
     */
    ODirSolver(std::string name);

    /** \brief
     * destructor
     */
    ~ODirSolver();

  protected:
    /** \brief
     * realisation of OEMSolver::solveSystem
     */
    int solveSystem(MatVecMultiplier<VectorType> *mv, VectorType *x, VectorType *b);

    /** \brief
     * realisation of OEMSolver::init
     */
    void init();

    /** \brief
     * realisation of OEMSolver::exit
     */
    void exit();

  private:
    // pointer to memory needed for solveSystem
    VectorType *Ad, *d, *d_old, *r, *h;
  };

}

#include "ODirSolver.hh"

#endif // AMDIS_ODIR_SOLVER_H
