// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file StandardProblemIteration.h */

#ifndef AMDIS_STANDARDPROBLEMITERATION_H
#define AMDIS_STANDARDPROBLEMITERATION_H

#include "ProblemIterationInterface.h"
#include "MemoryManager.h"

namespace AMDiS {

  class AdaptInfo;
  class ProblemStatBase;

  // ============================================================================
  // ===== class StandardProblemIteration =======================================
  // ============================================================================

  /** \brief
   * A master problem for a single non coupled problem.
   */
  class StandardProblemIteration : public ProblemIterationInterface
  {
  public:
    MEMORY_MANAGED(StandardProblemIteration);

    /** \brief
     * constructor
     */
    StandardProblemIteration(ProblemStatBase *problem)
      : problem_(problem)
    {};

    virtual ~StandardProblemIteration() {};

    /** \brief
     * implementation of \ref ProblemIterationIterface::beginIteration()
     */
    virtual void beginIteration(AdaptInfo *adaptInfo);

    /** \brief
     * implementation of \ref ProblemIterationInterface::oneIteration()
     */
    virtual Flag oneIteration(AdaptInfo *adaptInfo, Flag toDo);

    /** \brief
     * implementation of \ref ProblemIterationInterface::endIteration()
     */
    virtual void endIteration(AdaptInfo *adaptInfo);

    /** \brief
     * implementation of \ref ProblemIterationInterface::getNumProblems()
     */
    int getNumProblems() { 
      return 1; 
    };

    /** \brief
     * implementation of \ref ProblemIterationInterface::getProblem(int)
     */
    ProblemStatBase *getProblem(int number = 0) {
      return problem_;
    };

    /** \brief
     * Returns the name of the problem.
     */
    virtual const std::string& getName();

    /** \brief
     * Function that serializes the problem plus information about the iteration.
     */
    virtual void serialize(std::ostream &out);

    /** \brief
     * Function that deserializes the problem plus information about the iteration.
     */
    virtual void deserialize(std::istream &in);


  protected:
    /** \brief
     * nested assemblage and mesh adaption
     */
    Flag buildAndAdapt(AdaptInfo *adaptInfo, Flag toDo);

  protected:
    /** \brief
     * the problem to solve
     */
    ProblemStatBase *problem_;

    /** \brief
     * info level
     */
    static int info_;
  };

}

#endif

