// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file SurfaceQuadrature.h */

#ifndef AMDIS_SURFACEQAUDRATURE_H
#define AMDIS_SURFACEQAUDRATURE_H

#include <list>

#include "FixVec.h"
#include "MemoryManager.h"
#include "Quadrature.h"

namespace AMDiS {

  // ============================================================================
  // ===== class SurfaceQuadrature ==============================================
  // ============================================================================

  /** 
   * \ingroup Integration
   *
   * \brief
   * Quadrature for element surfaces. Uses the functionality of the standard
   * Quadrature class for dim-1 but calculates new barycentric coordinates
   * for the element sides.
   */
  class SurfaceQuadrature : public Quadrature
  {
  public:
    MEMORY_MANAGED(SurfaceQuadrature);

    /** \brief
     * Returns a SurfaceQuadrature for given dim and degree of the given
     * side number.
     */
    //static SurfaceQuadrature* 
    //  provideSurfaceQuadrature(int dim, 
    //			     int degree,
    //			     DimVec<double> *coords);

    /** \brief
     * Constructs a SurfaceQuadrature based on a standard Quadrature of dim-1.
     */
    SurfaceQuadrature(Quadrature *quad, VectorOfFixVecs<DimVec<double> >& coords);

    /** \brief
     * Destructor.
     */
    ~SurfaceQuadrature()
    {
    };

    /** \brief
     * Adapts SurfaceQuadrature to \ref coords.
     */
    void scaleSurfaceQuadrature(VectorOfFixVecs<DimVec<double> > &coords);

  protected:
    /** \brief
     * Pointer to the original quadrature
     */
    Quadrature *quad_;

    VectorOfFixVecs<DimVec<double> > coords_;

    /** \brief
     * List of all existing surface quadratures. Used by 
     * \ref provideSurfaceQuadrature(). If the needed quadrature is not in the
     * list, a new list entry is created.
     */
    //static std::list<SurfaceQuadrature*> surfaceQuadratureList;
  };

}

#endif // AMDIS_SURFACEQAUDRATURE_H
