#include <stdio.h>
#include <string>
#include <fstream>
#include <cmath>

#include "VtkWriter.h"
#include "DataCollector.h"
#include "DOFVector.h"
#include "SurfaceRegion_ED.h"
#include "ElementRegion_ED.h"

namespace AMDiS { 

  int VtkWriter::writeFile(const char *name)
  {
#ifdef HAVE_BOOST
    
    boost::iostreams::filtering_ostream file;
    switch (compress) {
    case GZIP:
      file.push(boost::iostreams::gzip_compressor());
      file.push(boost::iostreams::file_descriptor_sink(strcat((char*)name, ".gz")));    
      break;
    case BZIP2:
      file.push(boost::iostreams::bzip2_compressor());
      file.push(boost::iostreams::file_descriptor_sink(strcat((char*)name, ".bz2")));
      break;
    default:
      file.push(boost::iostreams::file_descriptor_sink(name));
      break;
    } 
    writeFile(file);

#else

    std::ofstream file;
    file.open(name);
    writeFile(file);    
    file.close();

#endif
      
    return 0;
  }

  int VtkWriter::updateAnimationFile(std::string valueFilename,
				     std::vector< std::string > *paraViewAnimationFrames,
				     const char *animationFilename)
  {
    size_t found = valueFilename.find_last_of("/\\");
    paraViewAnimationFrames->push_back(valueFilename.substr(found + 1));

    std::ofstream file;
    file.open(animationFilename);

    file << "<?xml version=\"1.0\"?>\n";
    file << "<VTKFile type=\"Collection\" version=\"0.1\" >"  << "\n";
    file << "<Collection>\n";

    int counter = 0;
    std::vector< std::string >::iterator it;
    for (it = paraViewAnimationFrames->begin(); 
	 it < paraViewAnimationFrames->end(); 
	 ++it, counter++) {
      file << "<DataSet timestep=\"" << counter
	   << "\" part=\"0\" file=\"" << (*it) << "\"/>\n";      
    }

    file << "</Collection>\n";
    file << "</VTKFile>\n";

    file.close();
    
    return 0;
  }


}
